import re

from pygments.lexer import bygroups, include, default
from pygments.lexers.javascript import JavascriptLexer
from pygments.token import Name, Operator, Punctuation, String, Text


# Use same tokens as `JavascriptLexer`, but with tags and attributes support
TOKENS = JavascriptLexer.tokens
TOKENS.update(
    {
        "jsx": [
            (
                r"(<)(/?)(>)",
                bygroups(Punctuation, Punctuation, Punctuation),
            ),  # JSXFragment <>|</>
            (r"(<)([\w]+)(\.?)", bygroups(Punctuation, Name.Tag, Punctuation), "tag"),
            (
                r"(<)(/)([\w]+)(>)",
                bygroups(Punctuation, Punctuation, Name.Tag, Punctuation),
            ),
            (
                r"(<)(/)([\w]+)",
                bygroups(Punctuation, Punctuation, Name.Tag),
                "fragment",
            ),  # Same for React.Context
        ],
        "tag": [
            (r"\s+", Text),
            (r"([\w]+\s*)(=)(\s*)", bygroups(Name.Attribute, Operator, Text), "attr"),
            (r"[{}]+", Punctuation),
            (r"[\w\.]+", Name.Attribute),
            (r"(/?)(\s*)(>)", bygroups(Punctuation, Text, Punctuation), "#pop"),
        ],
        "fragment": [
            (r"(.)([\w]+)", bygroups(Punctuation, Name.Attribute)),
            (r"(>)", bygroups(Punctuation), "#pop"),
        ],
        "attr": [
            ("{", Punctuation, "expression"),
            ('".*?"', String, "#pop"),
            ("'.*?'", String, "#pop"),
            default("#pop"),
        ],
        "expression": [
            ("{", Punctuation, "#push"),
            ("}", Punctuation, "#pop"),
            include("root"),
        ],
    }
)
TOKENS["root"].insert(0, include("jsx"))


class JsxLexer(JavascriptLexer):
    name = "react"
    aliases = ["jsx", "react"]
    filenames = ["*.jsx", "*.react"]
    mimetypes = ["text/jsx", "text/typescript-jsx"]

    flags = re.MULTILINE | re.DOTALL | re.UNICODE

    tokens = TOKENS



from pygments.token import Token


TOKENS = [
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "isOldEnough"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "("),
    (Token.Name.Other, "value"),
    (Token.Punctuation, ","),
    (Token.Text, " "),
    (Token.Name.Other, "ownProps"),
    (Token.Punctuation, ")"),
    (Token.Text, " "),
    (Token.Punctuation, "=>"),
    (Token.Text, " "),
    (Token.Punctuation, "{"),
    (Token.Text, "\n    "),
    (Token.Keyword, "if"),
    (Token.Text, " "),
    (Token.Punctuation, "("),
    (Token.Name.Builtin, "parseInt"),
    (Token.Punctuation, "("),
    (Token.Name.Other, "value"),
    (Token.Punctuation, ","),
    (Token.Text, " "),
    (Token.Literal.Number.Integer, "10"),
    (Token.Punctuation, ")"),
    (Token.Text, " "),
    (Token.Operator, "<"),
    (Token.Text, " "),
    (Token.Literal.Number.Integer, "14"),
    (Token.Punctuation, ")"),
    (Token.Text, " "),
    (Token.Punctuation, "{"),
    (Token.Text, "\n        "),
    (Token.Keyword, "return"),
    (Token.Text, " "),
    (Token.Literal.String.Double, '"Only 14yo and older can register to the site."'),
    (Token.Text, "\n    "),
    (Token.Punctuation, "}"),
    (Token.Text, "\n"),
    (Token.Punctuation, "}"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n\n"),
    (Token.Comment.Single, "// functional component\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "BlogTitle"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "("),
    (Token.Punctuation, "{"),
    (Token.Text, " "),
    (Token.Name.Other, "children"),
    (Token.Text, " "),
    (Token.Punctuation, "}"),
    (Token.Punctuation, ")"),
    (Token.Text, " "),
    (Token.Punctuation, "=>"),
    (Token.Text, " "),
    (Token.Punctuation, "("),
    (Token.Text, "\n  "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "h3"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "{"),
    (Token.Name.Other, "children"),
    (Token.Punctuation, "}"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "h3"),
    (Token.Punctuation, ">"),
    (Token.Text, "\n"),
    (Token.Punctuation, ")"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n\n"),
    (Token.Comment.Single, "// class component\n"),
    (Token.Keyword.Reserved, "class"),
    (Token.Text, " "),
    (Token.Name.Other, "BlogPost"),
    (Token.Text, " "),
    (Token.Keyword.Reserved, "extends"),
    (Token.Text, " "),
    (Token.Name.Other, "React"),
    (Token.Punctuation, "."),
    (Token.Name.Other, "Component"),
    (Token.Text, " "),
    (Token.Punctuation, "{"),
    (Token.Text, "\n  "),
    (Token.Name.Other, "renderTitle"),
    (Token.Punctuation, "("),
    (Token.Name.Other, "title"),
    (Token.Punctuation, ")"),
    (Token.Text, " "),
    (Token.Punctuation, "{"),
    (Token.Text, "\n    "),
    (Token.Keyword, "return"),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "BlogTitle"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "{"),
    (Token.Name.Other, "title"),
    (Token.Punctuation, "}"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "BlogTitle"),
    (Token.Punctuation, ">"),
    (Token.Text, "\n  "),
    (Token.Punctuation, "}"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n  "),
    (Token.Name.Other, "render"),
    (Token.Punctuation, "("),
    (Token.Punctuation, ")"),
    (Token.Text, " "),
    (Token.Punctuation, "{"),
    (Token.Text, "\n    "),
    (Token.Keyword, "return"),
    (Token.Text, " "),
    (Token.Punctuation, "("),
    (Token.Text, "\n    "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "div"),
    (Token.Text, " "),
    (Token.Name.Attribute, "className"),
    (Token.Operator, "="),
    (Token.Literal.String, '"blog-body"'),
    (Token.Punctuation, ">"),
    (Token.Text, "\n      "),
    (Token.Punctuation, "{"),
    (Token.Keyword, "this"),
    (Token.Punctuation, "."),
    (Token.Name.Other, "renderTitle"),
    (Token.Punctuation, "("),
    (Token.Keyword, "this"),
    (Token.Punctuation, "."),
    (Token.Name.Other, "props"),
    (Token.Punctuation, "."),
    (Token.Name.Other, "title"),
    (Token.Punctuation, ")"),
    (Token.Punctuation, "}"),
    (Token.Text, "\n      "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "p"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "{"),
    (Token.Keyword, "this"),
    (Token.Punctuation, "."),
    (Token.Name.Other, "props"),
    (Token.Punctuation, "."),
    (Token.Name.Other, "body"),
    (Token.Punctuation, "}"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "p"),
    (Token.Punctuation, ">"),
    (Token.Text, "\n      "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "CustomComponent"),
    (Token.Punctuation, ">"),
    (Token.Name.Other, "text"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "CustomComponent"),
    (Token.Punctuation, ">"),
    (Token.Text, "\n      "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "input"),
    (Token.Text, " "),
    (Token.Name.Attribute, "type"),
    (Token.Operator, "="),
    (Token.Literal.String, '"text"'),
    (Token.Text, " "),
    (Token.Punctuation, "{"),
    (Token.Name.Attribute, "...props.inputProps"),
    (Token.Punctuation, "}"),
    (Token.Text, " "),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Text, "\n    "),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "div"),
    (Token.Punctuation, ">"),
    (Token.Text, "\n    "),
    (Token.Punctuation, ")"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n  "),
    (Token.Punctuation, "}"),
    (Token.Text, "\n"),
    (Token.Punctuation, "}"),
    (Token.Text, "\n\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "body"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Literal.String.Double, '"Hello World!"'),
    (Token.Punctuation, ";"),
    (Token.Text, "\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "blogNode"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "BlogPost"),
    (Token.Text, " "),
    (Token.Name.Attribute, "title"),
    (Token.Operator, "="),
    (Token.Literal.String, '"What\'s going on?"'),
    (Token.Text, " "),
    (Token.Name.Attribute, "body"),
    (Token.Operator, "="),
    (Token.Punctuation, "{"),
    (Token.Name.Other, "body"),
    (Token.Punctuation, "}"),
    (Token.Text, " "),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n"),
    (Token.Comment.Single, "// some comment. Tags shouldn't be lexed in here\n"),
    (Token.Comment.Single, '// <div class="blog-body">\n'),
    (Token.Comment.Single, "// <h3>What's going on?</h3>\n"),
    (Token.Comment.Single, "// <p>Hello World!</p>\n"),
    (Token.Comment.Single, "// </div>\n"),
    (Token.Text, "\n"),
    (
        Token.Comment.Multiline,
        "/*\n  Some comment. Tags shouldn't be lexed in here either\n  <div class=\"blog-body\">\n  <h3>What's going on?</h3>\n  <p>Hello World!</p>\n  </div>\n*/",
    ),
    (Token.Text, "\n\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "shortSyntaxfragmentEmptyBody"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "shortSyntaxfragmentFullBody"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "div"),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "reactDotFragment"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "React"),
    (Token.Punctuation, "."),
    (Token.Name.Attribute, "Fragment"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "div"),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "React"),
    (Token.Punctuation, "."),
    (Token.Name.Attribute, "Fragment"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "reactDotContext"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "Context"),
    (Token.Punctuation, "."),
    (Token.Name.Attribute, "Provider"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "div"),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "Context"),
    (Token.Punctuation, "."),
    (Token.Name.Attribute, "Provider"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n\n"),
    (Token.Keyword.Reserved, "const"),
    (Token.Text, " "),
    (Token.Name.Other, "reactDotContextValue"),
    (Token.Text, " "),
    (Token.Operator, "="),
    (Token.Text, " "),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "Context"),
    (Token.Punctuation, "."),
    (Token.Name.Attribute, "Provider"),
    (Token.Text, " "),
    (Token.Name.Attribute, "value"),
    (Token.Operator, "="),
    (Token.Literal.String, '"Hello"'),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Name.Tag, "div"),
    (Token.Punctuation, "/"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, "<"),
    (Token.Punctuation, "/"),
    (Token.Name.Tag, "Context"),
    (Token.Punctuation, "."),
    (Token.Name.Attribute, "Provider"),
    (Token.Punctuation, ">"),
    (Token.Punctuation, ";"),
    (Token.Text, "\n"),
]



import io
import os

from setuptools import setup, find_packages


VERSION = "0.0.8"
here = os.path.abspath(os.path.dirname(__file__))
README = io.open(os.path.join(here, "README.rst"), encoding="utf8").read()

setup(
    name="jsx-lexer",
    description="A JSX lexer for Pygments",
    long_description=README,
    version=VERSION,
    url="https://github.com/fcurella/jsx-lexer",
    author="Flavio Curella",
    author_email="flavio.curella@gmail.com",
    classifiers=[
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
    keywords="pygments highlight jsx react",
    install_requires=["Pygments >= 2.1"],
    test_suite="tests",
    license="MIT License",
    packages=find_packages(exclude=["docs", "tests", "tests.*"]),
    entry_points="""
        [pygments.lexers]
        jsx=jsx:JsxLexer
    """,
)



import os
import re
from unittest import TestCase

from pygments import lexers
from pygments.token import Token
from jsx import lexer as lexer_mod
from jsx.lexer import JsxLexer

from .tokens import TOKENS as expected_tokens


CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))


lexer = lexers.load_lexer_from_file(lexer_mod.__file__, "JsxLexer")

with open(os.path.join(CURRENT_DIR, "example.jsx"), "r") as fh:
    text = fh.read()


class JsxLexerTestCase(TestCase):
    def __filter_tokens(self, tokens):
        space = re.compile("[ \n]+")
        return [i for i in tokens if not space.match(i[1]) and not i[1] == ""]

    def test_guess_lexer_for_filename(self):
        guessed_lexer = lexers.guess_lexer_for_filename("test.jsx", text)
        self.assertEqual(guessed_lexer.name, JsxLexer.name)

    def test_get_lexer_by_name(self):
        lexer = lexers.get_lexer_by_name("jsx")
        self.assertEqual(lexer.name, JsxLexer.name)

    def test_get_tokens(self):
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens(text)
        self.assertEqual([i for i in tokens], expected_tokens)

    def test_lexing_object_attribute(self):
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens(
            """
            <div style={{ color: 'red' }} />
        """
        )

        self.assertEqual(
            self.__filter_tokens(tokens),
            [
                (Token.Punctuation, "<"),
                (Token.Name.Tag, "div"),
                (Token.Name.Attribute, "style"),
                (Token.Operator, "="),
                (Token.Punctuation, "{"),
                (Token.Punctuation, "{"),
                (Token.Name.Other, "color"),
                (Token.Operator, ":"),
                (Token.Literal.String.Single, "'red'"),
                (Token.Punctuation, "}"),
                (Token.Punctuation, "}"),
                (Token.Punctuation, "/"),
                (Token.Punctuation, ">"),
            ],
        )

    def test_lexing_multiple_attributes(self):
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens(
            """
            <User name={'jhon'} last={'doe'} />
        """
        )

        self.assertEqual(
            self.__filter_tokens(tokens),
            [
                (Token.Punctuation, "<"),
                (Token.Name.Tag, "User"),
                (Token.Name.Attribute, "name"),
                (Token.Operator, "="),
                (Token.Punctuation, "{"),
                (Token.Literal.String.Single, "'jhon'"),
                (Token.Punctuation, "}"),
                (Token.Name.Attribute, "last"),
                (Token.Operator, "="),
                (Token.Punctuation, "{"),
                (Token.Literal.String.Single, "'doe'"),
                (Token.Punctuation, "}"),
                (Token.Punctuation, "/"),
                (Token.Punctuation, ">"),
            ],
        )

    def test_lexing_arrow_function_attribute(self):
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens(
            """
            <button onClick={e => e.preventDefault ()} />
        """
        )

        self.assertEqual(
            self.__filter_tokens(tokens),
            [
                (Token.Punctuation, "<"),
                (Token.Name.Tag, "button"),
                (Token.Name.Attribute, "onClick"),
                (Token.Operator, "="),
                (Token.Punctuation, "{"),
                (Token.Name.Other, "e"),
                (Token.Punctuation, "=>"),
                (Token.Name.Other, "e"),
                (Token.Punctuation, "."),
                (Token.Name.Other, "preventDefault"),
                (Token.Punctuation, "("),
                (Token.Punctuation, ")"),
                (Token.Punctuation, "}"),
                (Token.Punctuation, "/"),
                (Token.Punctuation, ">"),
            ],
        )

    def test_lexing_function_returing_jsx(self):
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens(
            """
            <Component wrapped={data => <User name={data.name} />} />
        """
        )

        self.assertEqual(
            self.__filter_tokens(tokens),
            [
                (Token.Punctuation, "<"),
                (Token.Name.Tag, "Component"),
                (Token.Name.Attribute, "wrapped"),
                (Token.Operator, "="),
                (Token.Punctuation, "{"),
                (Token.Name.Other, "data"),
                (Token.Punctuation, "=>"),
                (Token.Punctuation, "<"),
                (Token.Name.Tag, "User"),
                (Token.Name.Attribute, "name"),
                (Token.Operator, "="),
                (Token.Punctuation, "{"),
                (Token.Name.Other, "data"),
                (Token.Punctuation, "."),
                (Token.Name.Other, "name"),
                (Token.Punctuation, "}"),
                (Token.Punctuation, "/"),
                (Token.Punctuation, ">"),
                (Token.Punctuation, "}"),
                (Token.Punctuation, "/"),
                (Token.Punctuation, ">"),
            ],
        )

    def test_lexing_short_syntax_fragments(self):
        """
        Testing <></> React Short Syntax JSXFragment
        see: `https://facebook.github.io/jsx/`
        JSXFragment :
            <> JSXChildrenopt </>
        """
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens("""<></>""")
        self.assertEqual(
            self.__filter_tokens(tokens),
            [
                (Token.Punctuation, "<"),
                (Token.Punctuation, ">"),
                (Token.Punctuation, "<"),
                (Token.Punctuation, "/"),
                (Token.Punctuation, ">"),
            ],
        )

    def test_lexing_full_fragments(self):
        """
        Testing <React.Fragment></React.Fragment>
        """
        lexer = lexers.get_lexer_by_name("jsx")
        tokens = lexer.get_tokens("""<React.Fragment></React.Fragment>""")
        self.assertEqual(
            self.__filter_tokens(tokens),
            [
                (Token.Punctuation, "<"),
                (Token.Name.Tag, "React"),
                (Token.Punctuation, "."),
                (Token.Name.Attribute, "Fragment"),
                (Token.Punctuation, ">"),
                (Token.Punctuation, "<"),
                (Token.Punctuation, "/"),
                (Token.Name.Tag, "React"),
                (Token.Punctuation, "."),
                (Token.Name.Attribute, "Fragment"),
                (Token.Punctuation, ">"),
            ],
        )
